# Copyright (c) 2021-2023 Alibaba Cloud Computing Ltd.
# SPDX-License-Identifier: MulanPSL-2.0

import os
import re

from collections import defaultdict
from agent.common.system import sysCommand


class ConfigFile:
    def __init__(self, path, pattern = r"^(.+)\s*=\s*(.+)"):
        self.path = path  
        self.pattern = pattern
    
    def readValue(self, key):
        with open(self.path, 'r') as f:
            content = f.read()
        
        for k,v in re.findall(self.pattern, content):
            if k.strip() == key.strip():
                return v
        return None
    
    def writeValue(self, key, value):
        sysCommand("echo {key} = {value} >> {path}".format(
            key = key,
            value = value,
            path = self.path
        ))
        
    def removeDuplicate(self):
        with open(self.path, 'r') as f:
            content = f.read()
        
        content_lines = [i for i in content.split('\n') if i != '']
        no_duplicate_dict = defaultdict(str)
        no_duplicate_content = ""
        
        for line in content_lines:
            if line.startswith("#"):
                no_duplicate_content += line + '\n'
                
            elif re.search(self.pattern, line):
                key = re.search(self.pattern, line).group(1)
                no_duplicate_dict[key] = line
        
        for _, line in no_duplicate_dict.items():
            no_duplicate_content += line + '\n'
        
        with open(self.path, 'w') as f:
            f.write(no_duplicate_content)

def glob(path):
    paths = _glob([path])
    return [p for p in paths if os.path.exists(p)]

def _glob(path_list:list):
    """ replace '*' wildcard in path and return all matched path

    """
    def _splitByWildcard(path):
        prefix   = '/'
        wildcard = ''
        suffix   = ''
        
        for _p in path.split('/'):
            if wildcard != '':
                suffix = os.path.join(suffix, _p)
            elif '*' in _p:
                wildcard = _p
            else:
                prefix = os.path.join(prefix, _p)
        return prefix, wildcard, suffix
    
    def _replaceWildcard(prefix, wildcard, suffix):
        if not os.path.exists(prefix):
            return []

        candidates = os.listdir(prefix)                        
        pattern = re.compile(re.sub(r"\*", ".*", wildcard))
        valid_candidate = [c for c in candidates if re.match(pattern, c)]
        
        result = []
        for c in valid_candidate:
            result.append(os.path.join(prefix, c, suffix))
        return result
    
    result = []
    for path in path_list:
        if "*" not in path:
            result.append(path)
        else:
            prefix, wildcard, suffix = _splitByWildcard(path)
            _res = _replaceWildcard(prefix, wildcard, suffix)
            result += _glob(_res)
    return result