# Copyright (c) 2021-2023 Alibaba Cloud Computing Ltd.
# SPDX-License-Identifier: MulanPSL-2.0

import os  
import logging

from agent.domain.base import BaseDomain
from agent.common.system import sysCommand

logger = logging.getLogger('common')

class Scheduler(BaseDomain):
    __domain__ = "scheduler"
    
    def __init__(self):
        super().__init__(domain_name = self.__domain__)
        param_list = [
            'sched_min_granularity_ns',    # /proc/sys/kernel/sched_min_granularity_ns
            'sched_latency_ns',            # /proc/sys/kernel/sched_latency_ns
            'sched_features',              # /proc/sys/kernel/sched_features(3183d=110001101111b)
            'sched_wakeup_granularity_ns', # /proc/sys/kernel/sched_wakeup_granularity_ns(4000000ns)
            'sched_child_runs_first',      # /proc/sys/kernel/sched_child_runs_first(0)
            'sched_cfs_bandwidth_slice_us',# /proc/sys/kernel/sched_cfs_bandwidth_slice_us(5000us)
            'sched_rt_period_us',          # /proc/sys/kernel/sched_rt_period_us(1000000us)
            'sched_rt_runtime_us',         # /proc/sys/kernel/sched_rt_runtime_us(950000us)
            'sched_compat_yield',          # /proc/sys/kernel/sched_compat_yield(0)
            'sched_migration_cost',        # /proc/sys/kernel/sched_migration_cost(500000ns)
            'sched_nr_migrate',            # /proc/sys/kernel/sched_nr_migrate(32)
            'sched_tunable_scaling',       # /proc/sys/kernel/sched_tunable_scaling(1)
            'sched_migration_cost_ns',     # /proc/sys/kernel/sched_migration_cost_ns
        ]
        self.available_param_list = [p for p in param_list if os.path.exists(os.path.join("/proc/sys/kernel", p))]
        if len(self.available_param_list) == 0:
            logger.warning("no param available in /proc/sys/kernel")
            raise Exception("no param available in /proc/sys/kernel")

    def vaild_param(self, param_name):
        return param_name in self.available_param_list
    
    def set_value(self, param_name:str, param_value):
        if param_value != None:
            sysCommand("echo {} > /proc/sys/kernel/{}".format(param_value, param_name))
    
    def get_value(self, param_name:str):
        return sysCommand("cat /proc/sys/kernel/{}".format(param_name))